// $Id: Sf2Smt.h,v 1.3 2007-06-14 21:29:48 babic Exp $
//
// Copyright (C) 2007. Domagoj Babic, see the LICENSE.

#include <iostream>
#include <vector>
#include <map>

// Warning: This parser performs absolutely no checking! So, make sure
// that your input is valid. (Try running Spear on it, Spear thorougly
// checks that the input is valid).
class Parser {
    typedef std::map<std::string, unsigned> SymbolMapTy;
    typedef SymbolMapTy::iterator iterator;

    SymbolMapTy Symbols;
    std::string Name;
    std::vector<std::string> Header;
    std::vector<std::string> Body;
    unsigned Cnt;

    // Parses only the name in declaration, for instance, for nm:i32
    // it returns nm.
    std::string parseVarDef(std::istream &IS);

    // Parses a constant and puts it into val
    void parseConstant(std::istream &IS, uint64_t &val);

    // Parses a list of operands until it hits the end of line.
    // The second operand is a vector of operands as <name,size in bits>
    // pairs.
    void parseOperands(std::istream &IS, 
        std::vector<std::pair<std::string, unsigned> > &Ops,
        bool EncodeConstantsDirectly = false);

    // Parses only the header.
    void parseHeader(std::istream &IS);

    // Parses and translates constraint and predicate lines
    void parseLine(std::istream &IS);

    // Encodes the operator as (operator operands) and performs required
    // conversions (bool <-> bitvector).
    std::string encodeOperator(std::string &Operator, 
        std::vector<std::pair<std::string, unsigned> > &Ops, 
        unsigned ResultBits);
public:

    Parser(const std::string &Nm) : Name(Nm), Cnt(0) {}

    // Prases the input.
    void parse(std::istream &IS);

    // Dumps the translated instance.
    friend std::ostream &operator<<(std::ostream&, const Parser&);
};
    
std::ostream &operator<<(std::ostream&, const Parser&);
